--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 29 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: TRIG_DAC		  																--
--	Description : 																					--
-- 	Top Level for Trigger Dac Control													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DAC_MAIN is
   port ( 
			 xCLK			 	: in    std_logic;
			 xCLK_15MHz	 	: in    std_logic;
			 xCLK_100Hz	 	: in    std_logic;
			 xCLK_10Hz	 	: in    std_logic;
			 xCLK_1Hz	 	: in    std_logic;
			 xCLR_ALL	 	: in    std_logic;
          xRCO    		: in    std_logic;
			 xTRIG_THRES	: in    std_logic_vector (11 downto 0);
			 xWBIAS			: in    std_logic_vector (11 downto 0);
			 xROVDD 			: out   std_logic_vector (15 downto 0);
			 SCLK 			: out   std_logic;
			 SYNC 			: out   std_logic;
			 DIN0 			: out   std_logic);
end DAC_MAIN;

architecture Behavioral of DAC_MAIN is
	signal xREFRESH	: std_logic;
	signal xBEGIN 		: std_logic;
	signal S_CLK	 	: std_logic;
	signal N_SYNC	 	: std_logic;
	signal DIN_0	 	: std_logic;
	signal xUPDATE	 	: std_logic;
	signal xEN_A	 	: std_logic;
	signal xEN_B	 	: std_logic;
	signal xEN_C	 	: std_logic; 
	signal xEN_D	 	: std_logic;
	signal xS			: std_logic_vector(1 downto 0);
	signal DAC_0_A		: std_logic_vector(15 downto 0);
	signal DAC_0_B		: std_logic_vector(15 downto 0);
	signal DAC_0_C		: std_logic_vector(15 downto 0);
	signal DAC_0_D		: std_logic_vector(15 downto 0);
	signal ROVDD		: std_logic_vector(15 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component OBUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------
   component DAC_TIMING
   port ( 
			 xCLK    	: in    std_logic;	
			 xREFRESH 	: in    std_logic;
			 xCLR_ALL 	: in    std_logic;			 
			 xSCLK	 	: out    std_logic;
			 xNSYNC	 	: out    std_logic;
			 xUPDATE	 	: out    std_logic;
			 xEN_A		: out    std_logic;
			 xEN_B		: out    std_logic;
			 xEN_C		: out    std_logic;
			 xEN_D		: out    std_logic;
			 xS 		  	: out   std_logic_vector (1 downto 0));
   end component;
--------------------------------------------------------------------------------
   component DAC_SERIALIZER
   port ( 
			 xCLK    	: in    std_logic;
			 xSCLK    	: in    std_logic;
			 xUPDATE   	: in    std_logic;
			 xEN_A    	: in    std_logic;
			 xEN_B    	: in    std_logic;
			 xEN_C    	: in    std_logic;
			 xEN_D    	: in    std_logic;
			 xS			: in    std_logic_vector (1 downto 0);
			 xDAC_A 		: in    std_logic_vector (15 downto 0);
			 xDAC_B 		: in    std_logic_vector (15 downto 0);
			 xDAC_C 		: in    std_logic_vector (15 downto 0);
			 xDAC_D 		: in    std_logic_vector (15 downto 0);
			 xD_OUT	 	: out   std_logic);	
   end component;
--------------------------------------------------------------------------------
   component RCO_MAIN
   port ( 
			 xCLK    		: in    std_logic;
			 xCLK_15MHz	 	: in    std_logic;
			 xREFRESH  		: in    std_logic;			 
          xRCO    		: in    std_logic;
			 xCLR_ALL 		: in    std_logic;	
			 xDEBUG 		  	: in    std_logic_vector (15 downto 0);
			 xROVDD 		  	: out   std_logic_vector (15 downto 0);
			 xROGND		  	: out   std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	xRCO_MAIN : RCO_MAIN 
	port map (
		xCLK 			=> xCLK,
		xCLK_15MHz	=> xCLK_15MHz,
		xREFRESH => xBEGIN,
		xRCO	 	=> xRCO,
		xDEBUG 	=> x"0000",
		xCLR_ALL => xCLR_ALL,
		xROVDD 	=> ROVDD,
		xROGND 	=> open);
--------------------------------------------------------------------------------
	DAC_0_A <= x"3" & ROVDD(11 downto 0);
	DAC_0_B <= x"7" & xTRIG_THRES(11 downto 0);
	DAC_0_C <= x"B" & xWBIAS(11 downto 0);
	DAC_0_D <= x"E" & x"548";
--------------------------------------------------------------------------------
	xBEGIN <= xCLK_100Hz;
--------------------------------------------------------------------------------
	xDAC_TIMING : DAC_TIMING 
	port map (
		xCLK 		=> xCLK_15MHz,
		xCLR_ALL => '0',
		xREFRESH => xBEGIN,
		xSCLK 	=> S_CLK,
		xNSYNC 	=> N_SYNC,
		xUPDATE 	=> xUPDATE,
		xEN_A 	=> xEN_A,
		xEN_B 	=> xEN_B,
		xEN_C 	=> xEN_C,
		xEN_D 	=> xEN_D,
		xS(1 downto 0)=> xS(1 downto 0));
--------------------------------------------------------------------------------	
	xDAC_SERIALIZER_0 : DAC_SERIALIZER 
	port map (
		xCLK => xCLK_15MHz,
		xSCLK => S_CLK,
		xUPDATE => xUPDATE,
		xEN_A => xEN_A,
		xEN_B => xEN_B,
		xEN_C => xEN_C,
		xEN_D => xEN_D,
		xS(1 downto 0)=> xS(1 downto 0),
		xDAC_A(15 downto 0)=> DAC_0_A(15 downto 0),
		xDAC_B(15 downto 0)=> DAC_0_B(15 downto 0),
		xDAC_C(15 downto 0)=> DAC_0_C(15 downto 0),
		xDAC_D(15 downto 0)=> DAC_0_D(15 downto 0),
		xD_OUT => DIN_0);
--------------------------------------------------------------------------------
	xBUF_ROVDD : BUF16 
	port map (
		I	=> ROVDD,
		O	=> xROVDD);	
--------------------------------------------------------------------------------
	xBUF_SCLK : OBUF 
	port map (
		I  => S_CLK,
		O  => SCLK);	
--------------------------------------------------------------------------------
	xBUF_SYNC : OBUF 
	port map (
		I  => N_SYNC,
		O  => SYNC);	
--------------------------------------------------------------------------------
	xBUF_DIN1 : OBUF 
	port map (
		I  => DIN_0,
		O  => DIN0);	
--------------------------------------------------------------------------------
end Behavioral;