--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 12 JULY 2007																			--
-- Project name: BLAB1 firmware																--
--	Module name: GCCx4		  																	--
--	Description : 																					--
-- 	Four times Gray Code Counter															--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity GCCx2v2 is
port ( 
			 xRAMP			: in    std_logic;
			 xLOCK			: in    std_logic;
			 xREF				: in	  std_logic_vector(11 downto 0);
			 xGCC1			: in	  std_logic_vector(11 downto 0);
			 xGCC2			: in	  std_logic_vector(11 downto 0);
			 xOUT				: out   std_logic_vector(15 downto 0));
end GCCx2v2;

architecture Behavioral of GCCx2v2 is
	signal LOCK1	: std_logic_vector(11 downto 0);	
	signal LOCK2	: std_logic_vector(11 downto 0);	
	signal TDC1		: std_logic_vector(11 downto 0);	
	signal TDC2		: std_logic_vector(11 downto 0);	
	signal SUM		: std_logic_vector(11 downto 0);	
	signal DIFF		: std_logic_vector(11 downto 0);	
	signal OUTPUT	: std_logic_vector(11 downto 0);	
	signal OUT_PUT	: std_logic_vector(15 downto 0);	
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------
	component GCCINV_12BIT
		port (
			CNT	: IN std_logic_VECTOR(11 downto 0);
			HITT	: OUT std_logic_VECTOR(11 downto 0));
		end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------	
	process(xLOCK,xRAMP)
	begin
		if xRAMP = '1' and rising_edge(xLOCK) then
			LOCK1 <= xGCC1;
			LOCK2 <= xGCC2;
		end if;
	end process;		
--------------------------------------------------------------------------------	
	xGCCINV_1 : GCCINV_12BIT port map (
		CNT	=> LOCK1,
		HITT	=> TDC1);	
--------------------------------------------------------------------------------			
	xGCCINV_2 : GCCINV_12BIT port map (
		CNT	=> LOCK2,
		HITT	=> TDC2);	
--------------------------------------------------------------------------------			
	SUM <= TDC1 + TDC2;
	DIFF <= x"FFF" - xREF;
--------------------------------------------------------------------------------			
	process(SUM,xREF)
	begin
		if SUM > xREF then
			OUTPUT <= SUM - xREF;
		else
			OUTPUT <= SUM + DIFF + 1;
		end if;
	end process;
--------------------------------------------------------------------------------
	OUT_PUT <= x"0" & OUTPUT;
--------------------------------------------------------------------------------
	xOUTPUT_BUFFER : BUF16 port map (
		I(15 downto 0)=> OUT_PUT(15 downto 0),
		O(15 downto 0)=> xOUT(15 downto 0));			
--------------------------------------------------------------------------------	
end Behavioral;