--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 29 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: TRIG_DAC		  																--
--	Description : 									   											--
-- 	Top Level for Trigger Dac Control													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MAIN_REG is
   port ( xDSTRB     	: in    std_logic;
          xMAIN_DATA  	: in    std_logic_vector (15 downto 0);
          xSEL    		: in    std_logic;
			 xR_WR    		: in    std_logic;
          xCLR_ALL  		: in    std_logic;
			 xPED_COL_ADDR : out   std_logic_vector (4 downto 0);
			 xPED_ROW_ADDR : out   std_logic_vector (2 downto 0);
			 xTHRES	      : out   std_logic_vector (11 downto 0);
			 xWBIAS 			: out   std_logic_vector (11 downto 0);
			 xTERM 			: out   std_logic_vector (2 downto 0);
          xEN_PED   		: out   std_logic;
			 xSGN				: out   std_logic);
end MAIN_REG;

architecture Behavioral of MAIN_REG is
	type STATE_TYPE is ( PHASE0, PHASE1, PHASE2); 
	signal STATE			: STATE_TYPE;
	signal SGN				: std_logic := '0';
	signal EN_PED			: std_logic := '0';
	signal PED_COL_ADDR 	: std_logic_vector (4 downto 0) := (others=>'0');
	signal PED_ROW_ADDR 	: std_logic_vector (2 downto 0) := (others=>'0');
	signal TERM		 		: std_logic_vector (2 downto 0);
	signal THRES			: std_logic_vector (11 downto 0) := (others=>'0');
	signal WBIAS			: std_logic_vector (11 downto 0) := (others=>'0');
	signal Locmd			: std_logic_vector (15 downto 0);
	signal Hicmd			: std_logic_vector (15 downto 0);
	signal DUMMY			: std_logic := '0';
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF12
    Port ( 
			  I   : in   std_logic_vector (11 downto 0);
			  O	: out  std_logic_vector (11 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF5
    Port ( 
			  I   : in   std_logic_vector (4 downto 0);
			  O	: out  std_logic_vector (4 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF3
    Port ( 
			  I   : in   std_logic_vector (2 downto 0);
			  O	: out  std_logic_vector (2 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	xBUF_WBIAS : BUF12 
	port map (
		I	=> WBIAS,
		O	=>	xWBIAS);
--------------------------------------------------------------------------------
	xBUF_THRES : BUF12 
	port map (
		I	=> THRES,
		O	=>	xTHRES);
--------------------------------------------------------------------------------
	xBUF_PED_COL_ADDR : BUF5 
	port map (
		I	=> PED_COL_ADDR,
		O	=>	xPED_COL_ADDR);
--------------------------------------------------------------------------------
	xBUF_PED_ROW_ADDR : BUF3 
	port map (
		I	=> PED_ROW_ADDR,
		O	=>	xPED_ROW_ADDR);
--------------------------------------------------------------------------------
	xBUF_TERM : BUF3 
	port map (
		I	=> TERM,
		O	=>	xTERM);
--------------------------------------------------------------------------------		
	xBUF_EN_PED : BUF 
	port map (
		I  => EN_PED,
		O  => xEN_PED);
--------------------------------------------------------------------------------		
	xBUF_SGN : BUF 
	port map (
		I  => SGN,
		O  => xSGN);
--------------------------------------------------------------------------------		
	process(xSEL,xR_WR,xCLR_ALL,xDSTRB)
	begin
		if xCLR_ALL = '1' then
			THRES 			<= (others=>'0');
			WBIAS 			<= (others=>'0');
			PED_COl_ADDR	<= (others=>'0');
			PED_ROW_ADDR	<= (others=>'0');
			TERM				<= "111";
			EN_PED 			<= '0';
			SGN	 			<= '0';
			DUMMY				<= '0';
			Locmd				<= (others=>'0');
			Hicmd				<= (others=>'0');
			STATE				<= PHASE0;
		elsif xSEL = '0' then
			Locmd				<= (others=>'0');
			Hicmd				<= (others=>'0');
			STATE				<= PHASE0;
		elsif rising_edge(xDSTRB)and xSEL = '1' and xR_WR = '1' then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when PHASE0 =>	
					Locmd	<= xMAIN_DATA;
					STATE	<= PHASE1;
--------------------------------------------------------------------------------	
				when PHASE1 =>	
					Hicmd	<= xMAIN_DATA;
					STATE	<= PHASE2;
--------------------------------------------------------------------------------	
				when PHASE2 =>	
					case Locmd(12 downto 0) is
--------------------------------------------------------------------------------	
-----------       Software People only care about this part :-p    -------------	
--------------------------------------------------------------------------------							
						when '0' & x"003" => --Load data to THRES			
							THRES <=	Hicmd(11 downto 0);	--LOCMD "XXX0-0000-0000-0011"
																	--HICMD "DDDD-DDDD-DDDD-DDDD" 

						when '0' & x"004" =>	--Load data to EN_PED
							EN_PED <= Hicmd(0); 				--LOCMD "XXX0-0000-0000-0100" 
																	--HICMD "XXXX-XXXX-XXXX-XXXD"
									
						when '0' & x"005" =>	--Load data to PED_ROW_ADDR
							PED_COL_ADDR      				--LOCMD "XXX0-0000-0000-0101" 
								<= Hicmd(8 downto 4);
							PED_ROW_ADDR						--HICMD "XXXX-DDDD-DDDD-XXXX"
								<= Hicmd(11 downto 9); 	
									
						when '0' & x"006" =>	--Load data to WBIAS
							WBIAS <=	Hicmd(11 downto 0);	--LOCMD "XXX0-0000-0000-0110"
																	--HICMD "DDDD-DDDD-DDDD-DDDD" 
																	
						when '0' & x"007" =>	--Load data to SGN
							SGN <= Hicmd(0);			  		--LOCMD "XXX0-0000-0000-0111"
																	--HICMD "XXXX-XXXX-XXXX-XXXD"

						when '0' & x"008" =>	--Load data to TERM
							TERM <= Hicmd(2 downto 0);		--LOCMD "XXX0-0000-0000-1000" 
																	--HICMD "XXXX-XXXX-XXXX-XDDD"	
																	
--------------------------------------------------------------------------------	
-----------       Software People stop caring at this point  ^_^   -------------	
--------------------------------------------------------------------------------	
						when others =>	
							DUMMY <= '1';
					end case;
				when others =>	STATE<=PHASE0;																
			end case;
		end if;
	end process;					
--------------------------------------------------------------------------------	
end Behavioral;