--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 29 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: TRIG_DAC		  																--
--	Description : 																					--
-- 	Top Level for Trigger Dac Control													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity RCO_MAIN is
   port ( 
			 xCLK    		: in    std_logic;
			 xCLK_15MHz	 	: in    std_logic;
			 xREFRESH  		: in    std_logic;			 
          xRCO    		: in    std_logic;
			 xPRCO    		: in    std_logic;
			 xCLR_ALL 		: in    std_logic;		
			 xDEBUG 		  	: in    std_logic_vector (15 downto 0);
			 xROVDD 		  	: out   std_logic_vector (15 downto 0);
			 xROGND		  	: out   std_logic_vector (15 downto 0));
end RCO_MAIN;

architecture Behavioral of RCO_MAIN is
	signal INT				: std_logic_vector(15 downto 0);
	signal COMPARE			: std_logic_vector(15 downto 0);
	signal SET_VDD			: std_logic_vector(11 downto 0);
	signal ROVDD			: std_logic_vector(15 downto 0);
	signal ROGND			: std_logic_vector(15 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------
	component RCO_INT
   port ( 
			 xCLK    		: in    std_logic;
			 xREFRESH  		: in    std_logic;			 
			 xRCO	 			: in    std_logic;
			 xINT 		  	: out   std_logic_vector (15 downto 0));
	end component;
--------------------------------------------------------------------------------
	component RCO_CALC
   port ( 
			 xREFRESH  		: in    std_logic;
			 xCOMPARE  		: in    std_logic_vector(15 downto 0);			 
          xSET_VDD    	: in    std_logic_vector(11 downto 0);
			 xINT	 		  	: in    std_logic_vector(15 downto 0);
			 xCLR_ALL 		: in    std_logic;			 
			 xROVDD		  	: out   std_logic_vector(15 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	--Sampling Speed  = COMPARE*SLOPE + Y_INT
	--SLOPE = 3.404056E-05
	--Y_INT = 4.254700E-02
	COMPARE <= x"34A3";--x"6947"
	SET_VDD <= x"E14";
--------------------------------------------------------------------------------
	xRCO_INT : RCO_INT 
	port map (
		xCLK 		=> xCLK_15MHz, 
		xRCO 		=> xRCO,
		xREFRESH => xREFRESH,
		xINT		=> INT);
--------------------------------------------------------------------------------
	xRCO_CALC : RCO_CALC 
	port map (
		xREFRESH => xREFRESH,
		xCOMPARE => COMPARE,
		xSET_VDD => SET_VDD,
		xINT 		=> INT,
		xCLR_ALL => xCLR_ALL,
		xROVDD 	=> ROVDD);
--------------------------------------------------------------------------------
	xBUF_ROVDD : BUF16 
	port map (
		I	=> ROVDD,
		O	=> xROVDD);	
--------------------------------------------------------------------------------
	ROGND <= x"0" & not(ROVDD(11 downto 0));
--------------------------------------------------------------------------------
	xBUF_ROGND : BUF16 
	port map (
		I	=> ROGND,
		O	=> xROGND);	
--------------------------------------------------------------------------------
end Behavioral;