--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 25 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: SCALER_TOP	  																	--
--	Description : 																					--
-- 	Top level of scaler module																--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity SCALER_TOP is
   port ( 
			 xCLK_1Hz	: in   std_logic;
			 xSOFT_TRIG	: in   std_logic;
			 xEXT_TRIG	: in   std_logic;
			 xDHIT		: in   std_logic;
          xRADDR     : in   std_logic_vector (4 downto 0);
			 xCLR_ALL	: in   std_logic;
			 xSCALER		: out  std_logic_vector (15 downto 0));
end SCALER_TOP;

architecture Behavioral of SCALER_TOP is
	signal CNT0				: std_logic_vector (15 downto 0);
	signal CNT1				: std_logic_vector (15 downto 0);
	signal CNT2				: std_logic_vector (15 downto 0);
	signal SCALER0			: std_logic_vector (15 downto 0);
	signal SCALER1			: std_logic_vector (15 downto 0);
	signal SCALER2			: std_logic_vector (15 downto 0);
	signal CNT_INT			: std_logic_vector (15 downto 0);	
	signal OUTPUT			: std_logic_vector (15 downto 0);	
	signal CLR_CNT			: std_logic;	
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component SCALER
   port ( 
			 xCLR_CNT	 	: in   std_logic;
			 xCOUNT 			: in   std_logic;
			 xOUT				: out   std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------
   component BUF16
      port ( I  : in    std_logic_vector(15 downto 0); 
             O  : out   std_logic_vector(15 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	process(xCLR_ALL,xCLK_1Hz)
	begin
		if xCLR_ALL = '1' then
			SCALER0 		<= x"0000";
			SCALER1 		<= x"0000";
			SCALER2 		<= x"0000";
			CNT_INT 		<= (others=>'0');
			CLR_CNT		<= '1';
		elsif rising_edge(xCLK_1Hz) then
			CLR_CNT		<= '0';
			CNT_INT <= CNT_INT + 1;
			if CNT_INT = 60 then
				SCALER0 	<= CNT0;
				SCALER1  <= CNT1;
				SCALER2 	<= CNT2;
				CLR_CNT	<= '1';
				CNT_INT 	<= (others=>'0');
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------			
	xSCALER_0 : SCALER 
	port map (
		xCOUNT	=> xDHIT,
		xOUT 		=> CNT0,
		xCLR_CNT => CLR_CNT);	
--------------------------------------------------------------------------------			
	xSCALER_1 : SCALER 
	port map (
		xCOUNT	=> xEXT_TRIG,
		xOUT 		=> CNT1,
		xCLR_CNT => CLR_CNT);	
--------------------------------------------------------------------------------			
	xSCALER_2 : SCALER 
	port map (
		xCOUNT	=> xSOFT_TRIG,
		xOUT 		=> CNT2,
		xCLR_CNT => CLR_CNT);	
--------------------------------------------------------------------------------			
	process(xRADDR)
	begin
		if xRADDR = 0 then
			OUTPUT <= SCALER0;
		elsif xRADDR = 1 then
			OUTPUT <= SCALER1;
		elsif xRADDR = 2 then
			OUTPUT <= SCALER2;
		else
			OUTPUT <= x"0000";
		end if;
	end process;
--------------------------------------------------------------------------------	
	xBUF_OUTPUT : BUF16 
	port map (
		I	=> OUTPUT,
		O	=> xSCALER);	
--------------------------------------------------------------------------------	
end Behavioral;