--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 22 OCT 2007																			--
-- Project name: BLAB1 firmware									 							--
--	Module name: BLAB1_RAM_TIMING																--
--	Description : 																					--
-- 	BLAB1 RAM timing firmware																--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TARGET_RAM_TIMING is
   port ( 
			 xCLK    		: in    std_logic;
			 xDONE	 		: in    std_logic;
			 xNRUN	 		: in    std_logic;
			 xRAMP_DONE		: in    std_logic;
			 xCLR_ALL 		: in    std_logic;	
			 xW_STRB 		: out    std_logic;
			 xSTART	 		: out    std_logic;
			 xRESET_RAMP	: out    std_logic;
			 xADDR 		  	: out   std_logic_vector (15 downto 0));
end TARGET_RAM_TIMING;

architecture Behavioral of TARGET_RAM_TIMING is
	signal ADDR			: std_logic_vector(19 downto 0);
	signal MAP_ADDR	: std_logic_vector(19 downto 0);
	signal CNT_A		: std_logic_vector(19 downto 0);
	signal CNT_B		: std_logic_vector(19 downto 0);
	signal W_STRB		: std_logic;
	signal RESET_RAMP	: std_logic;
	signal START		: std_logic;
	signal xWAIT		: std_logic_vector(3 downto 0);
	signal STRB			: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------
	process(xCLR_ALL,xDONE,xRAMP_DONE,xCLK)
	begin
		if xCLR_ALL = '1' or xDONE = '1' or xRAMP_DONE = '0' then
			W_STRB <= '1';
			xWAIT  <= x"0";
		elsif xRAMP_DONE = '1' and rising_edge(xCLK) then
			if xWAIT = 13 then
				if W_STRB = '1' then
					W_STRB <= '0';
				else
					W_STRB <= '1';
				end if;
			else 
				W_STRB <= '1';
				xWAIT  <= xWAIT + 1;
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------
	xBUF_W_STRB : BUF 
	port map (
		I  => W_STRB,
		O  => xW_STRB);
--------------------------------------------------------------------------------
	xBUF_STRB : BUF 
	port map (
		I  => W_STRB,
		O  => STRB);		
--------------------------------------------------------------------------------
	process(W_STRB,xDONE,xCLR_ALL,xNRUN,START)
	begin
		if xDONE = '1' or xCLR_ALL = '1' then
			CNT_A <= (others=>'0');
		elsif rising_edge(W_STRB) and xNRUN = '1' and START = '0' then
			CNT_A <= CNT_A + 1;
		end if;
	end process;
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if falling_edge(xCLK) then
			CNT_B <= CNT_A;
		end if;	
	end process;
--------------------------------------------------------------------------------
	xBUF_ADDR : BUF16 
	port map (
		I	=> CNT_A(15 downto 0),
		O	=> xADDR);		
--------------------------------------------------------------------------------
	process(CNT_A)
	begin	
		if xRAMP_DONE = '0' then
			RESET_RAMP <= '0';
		elsif falling_edge(CNT_A(3)) then --16 pix at a time
			RESET_RAMP <= '1';
		end if;
	end process;
--------------------------------------------------------------------------------
	process(CNT_A,xCLR_ALL,xDONE)
	begin	
		if xCLR_ALL = '1' or xDONE = '1' then
				START <= '0';                  -- 8xCH (top and bottom) readout out at the same time
			elsif falling_edge(CNT_A(11)) then --(8xCH)*(16xPIX)*(32xDEEP) = 512
				START <= '1';
		end if;
	end process;
--------------------------------------------------------------------------------			
	xBUF_START : BUF 
	port map (
		I  => START,
		O  => xSTART);	
--------------------------------------------------------------------------------			
	xBUF_RESET_RAMP : BUF 
	port map (
		I  => RESET_RAMP,
		O  => xRESET_RAMP);	
--------------------------------------------------------------------------------			
end Behavioral;