--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 27 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: CH_TRIG		  																	--
--	Description : 																					--
-- 	Single Channel Trigger module															--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TARGET_TSA_CTRL is
   port ( 
			 xCLK		 			: in   std_logic;
			 xCLR_ALL 			: in   std_logic;
			 xDONE	 			: in   std_logic;
			 xTRIG 				: in   std_logic;
			 xNRUN 				: in   std_logic;
			 xMEM_LOCK			: out  std_logic;
			 xLOCK_COL			: out  std_logic_vector(4 downto 0);
			 xLOCK_ROW			: out  std_logic_vector(2 downto 0);
			 xTTSA				: out  std_logic_vector (3 downto 0);
			 xBTSA				: out  std_logic_vector (3 downto 0);
			 xTSA_SEL			: out  std_logic_vector(1 downto 0);
			 xTRACK_COL			: out  std_logic_vector(4 downto 0);
			 xADC_HIT			: out  std_logic);
end TARGET_TSA_CTRL;

architecture Behavioral of TARGET_TSA_CTRL is
	signal MAIN_CNT		: std_logic_vector(15 downto 0) := x"0000";
   signal TRACK_COL     : std_logic_vector(4 downto 0);
   signal TRACK_ROW     : std_logic_vector(2 downto 0);
   signal LOCK_COL      : std_logic_vector(4 downto 0);
   signal LOCK_ROW      : std_logic_vector(2 downto 0);
	signal MEM_LOCK		: std_logic;  
	signal ADC_HIT			: std_logic;  
	signal TSA_STRB      : std_logic_vector(1 downto 0);
	signal TSA_S		  	: std_logic_vector(7 downto 0);
	signal EN			  	: std_logic_vector(7 downto 0);
	signal TTSA		  	   : std_logic_vector(3 downto 0);
	signal BTSA		  	   : std_logic_vector(3 downto 0);
	signal STRB		      : std_logic_vector(1 downto 0);
	signal STRB0	      : std_logic_vector(1 downto 0);
	signal STRB1	      : std_logic_vector(1 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------	
	component TRIG_ONESHOT
   port ( 
			 xCLK			: in    std_logic;
			 xIN			: in    std_logic;
			 xOUT			: out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component TSA_32CYCLE_ONESHOT
   port ( 
			 xEN					: in   std_logic;
			 xCLK		 			: in   std_logic;
			 xTSA_STRB			: out  std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF
    Port ( 
			  I   : in   std_logic;
			  O	: out  std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF2
    Port ( 
			  I   : in   std_logic_vector (1 downto 0);
			  O	: out  std_logic_vector (1 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF3
    Port ( 
			  I   : in   std_logic_vector (2 downto 0);
			  O	: out  std_logic_vector (2 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF4
    Port ( 
			  I   : in   std_logic_vector (3 downto 0);
			  O	: out  std_logic_vector (3 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF5
    Port ( 
			  I   : in   std_logic_vector (4 downto 0);
			  O	: out  std_logic_vector (4 downto 0));
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if rising_edge(xCLK) then
			MAIN_CNT <= MAIN_CNT + 1;
		end if;
	end process;
--------------------------------------------------------------------------------
	TRACK_COL <= MAIN_CNT(3 downto 0) & '0';
	TRACK_ROW <= MAIN_CNT(6 downto 4);
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if xCLR_ALL = '1' or xDONE = '1' then
			MEM_LOCK <= '0';
		elsif falling_edge(xCLK) and xNRUN = '0' then
			if TRACK_COL = LOCK_COL and TRACK_ROW = LOCK_ROW  then
				MEM_LOCK <= '0';
			elsif xTRIG = '1' and MEM_LOCK = '0' then
				LOCK_COL <= TRACK_COL;
				LOCK_ROW <= TRACK_ROW;
				MEM_LOCK <= '1';		
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------
	xTRIG_ONESHOT : TRIG_ONESHOT 
	port map (
		xCLK	=> xCLK, 
		xIN	=> xTRIG, 
		xOUT	=> ADC_HIT);
--------------------------------------------------------------------------------
	xBUF_LOCK_COL : BUF5
	port map (
		I	=> LOCK_COL, 
		O	=> xLOCK_COL);	
--------------------------------------------------------------------------------
	xBUF_LOCK_ROW : BUF3 
	port map (
		I	=> LOCK_ROW, 
		O	=> xLOCK_ROW);	
--------------------------------------------------------------------------------
	xBUF_MEM_LOCK : BUF 
	port map (
		I	=> MEM_LOCK, 
		O	=> xMEM_LOCK);			
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if xNRUN = '1' then
			STRB <= "00";
		elsif falling_edge(xCLK) then
			if TRACK_COL > 9 then
				if TRACK_ROW(0) = '1' then
					STRB <= "01";
				else
					STRB <= "10";
				end if;
			else
				STRB <= "00";
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if rising_edge(xCLK) then		
			STRB0 <= STRB;
		end if;
	end process;	
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if falling_edge(xCLK) then		
			STRB1 <= STRB0;
		end if;
	end process;	
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if rising_edge(xCLK) then		
			TSA_STRB <= STRB1;
		end if;
	end process;	
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if falling_edge(xCLK) then
			if TRACK_ROW = 0 then
				EN <= "00000001";
			elsif TRACK_ROW = 1 then
				EN <= "00000010";
			elsif TRACK_ROW = 2 then
				EN <= "00000100";
			elsif TRACK_ROW = 3 then
				EN <= "00001000";
			elsif TRACK_ROW = 4 then
				EN <= "00010000";
			elsif TRACK_ROW = 5 then
				EN <= "00100000";
			elsif TRACK_ROW = 6 then
				EN <= "01000000";
			elsif TRACK_ROW = 7 then
				EN <= "10000000";
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT0 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(0), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(0));		
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT1 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(1), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(1));		
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT2 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(2), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(2));		
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT3 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(3), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(3));		
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT4 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(4), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(4));		
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT5 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(5), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(5));		
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT6 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(6), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(6));		
--------------------------------------------------------------------------------
	xTSA_32CYCLE_ONESHOT7 : TSA_32CYCLE_ONESHOT 
	port map (
		xEN			=> EN(7), 
		xCLK			=> xCLK, 
		xTSA_STRB	=> TSA_S(7));		
--------------------------------------------------------------------------------
	TTSA <= TSA_S(6) & TSA_S(4) & TSA_S(2) & TSA_S(0);
	BTSA <= TSA_S(7) & TSA_S(5) & TSA_S(3) & TSA_S(1);
--------------------------------------------------------------------------------
	xBUF_TTSA : BUF4 
	port map (
		I	=> TTSA, 
		O	=> xTTSA);
--------------------------------------------------------------------------------
	xBUF_BTSA : BUF4 
	port map (
		I	=> BTSA, 
		O	=> xBTSA);		
--------------------------------------------------------------------------------
	xBUF_TSA_STRB : BUF2 
	port map (
		I	=> TSA_STRB, 
		O	=> xTSA_SEL);	
--------------------------------------------------------------------------------
	xBUF_TRACK_COL : BUF5 
	port map (
		I	=> TRACK_COL, 
		O	=> xTRACK_COL);	
--------------------------------------------------------------------------------
	xBUF_ADC_HIT : BUF 
	port map (
		I	=> ADC_HIT, 
		O	=> xADC_HIT);			
--------------------------------------------------------------------------------
end Behavioral;