--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 25 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: DAC_SERIALIZER  																--
--	Description : 																					--
-- 	DAC serialization module																--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DAC_SERIALIZER is
   port ( 
			 xCLK    	: in    std_logic;
			 xSCLK    	: in    std_logic;
			 xUPDATE   	: in    std_logic;
			 xEN_A    	: in    std_logic;
			 xEN_B    	: in    std_logic;
			 xEN_C    	: in    std_logic;
			 xEN_D    	: in    std_logic;
			 xS			: in    std_logic_vector (1 downto 0);
			 xDAC_A 		: in    std_logic_vector (15 downto 0);
			 xDAC_B 		: in    std_logic_vector (15 downto 0);
			 xDAC_C 		: in    std_logic_vector (15 downto 0);
			 xDAC_D 		: in    std_logic_vector (15 downto 0);
			 xD_OUT	 	: out   std_logic);	
end DAC_SERIALIZER;

architecture Behavioral of DAC_SERIALIZER is
	type STATE_TYPE is ( DAT_ELSE, 
								DAT0,DAT1,DAT2,DAT3,
								DAT4,DAT5,DAT6,DAT7,
								DAT8,DAT9,DAT10,DAT11,
								DAT12,DAT13,DAT14);
	signal STATE 		   : STATE_TYPE;
	signal D_OUT			: std_logic;
	signal xSTART			: std_logic;
	signal xDATA			: std_logic_vector(15 downto 0);	
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	xBUF_D_OUT : BUF 
	port map (
		I  => D_OUT,
		O  => xD_OUT);		
--------------------------------------------------------------------------------	
	process(xEN_A,xEN_B,xEN_C,xEN_D)
	begin
		if xEN_A = '1' then
			xDATA(15 downto 0) <= xDAC_A(15 downto 0);
		elsif xEN_B = '1' then
			xDATA(15 downto 0) <= xDAC_B(15 downto 0);
		elsif xEN_C = '1' then
			xDATA(15 downto 0) <= xDAC_C(15 downto 0);
		elsif xEN_D = '1' then
			xDATA(15 downto 0) <= xDAC_D(15 downto 0);
		else
			xDATA(15 downto 0) <= (others=>'0');
		end if;
	end process;
--------------------------------------------------------------------------------	
	process(xCLK)
	begin
		if falling_edge(xCLK) then
			xSTART <= xUPDATE;
		end if;
	end process;
--------------------------------------------------------------------------------	
	process(xSCLK,xSTART)
	begin
		if xSTART = '0' then
			D_OUT <= xDATA(15);
			STATE <= DAT14;
		elsif rising_edge(xSCLK) then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when DAT14 =>			
					D_OUT <= xDATA(14);
					STATE <= DAT13;	
--------------------------------------------------------------------------------	
				when DAT13 =>			
					D_OUT <= xDATA(13);
					STATE <= DAT12;	
--------------------------------------------------------------------------------	
				when DAT12 =>			
					D_OUT <= xDATA(12);
					STATE <= DAT11;	
--------------------------------------------------------------------------------	
				when DAT11 =>			
					D_OUT <= xDATA(11);
					STATE <= DAT10;	
--------------------------------------------------------------------------------	
				when DAT10 =>			
					D_OUT <= xDATA(10);
					STATE <= DAT9;	
--------------------------------------------------------------------------------	
				when DAT9 =>			
					D_OUT <= xDATA(9);
					STATE <= DAT8;	
--------------------------------------------------------------------------------	
				when DAT8 =>			
					D_OUT <= xDATA(8);
					STATE <= DAT7;	
--------------------------------------------------------------------------------	
				when DAT7 =>			
					D_OUT <= xDATA(7);
					STATE <= DAT6;	
--------------------------------------------------------------------------------	
				when DAT6 =>			
					D_OUT <= xDATA(6);
					STATE <= DAT5;	
--------------------------------------------------------------------------------	
				when DAT5 =>			
					D_OUT <= xDATA(5);
					STATE <= DAT4;	
--------------------------------------------------------------------------------	
				when DAT4 =>			
					D_OUT <= xDATA(4);
					STATE <= DAT3;	
--------------------------------------------------------------------------------	
				when DAT3 =>			
					D_OUT <= xDATA(3);
					STATE <= DAT2;	
--------------------------------------------------------------------------------	
				when DAT2 =>			
					D_OUT <= xDATA(2);
					STATE <= DAT1;	
--------------------------------------------------------------------------------	
				when DAT1 =>			
					D_OUT <= xDATA(1);
					STATE <= DAT0;	
--------------------------------------------------------------------------------	
				when DAT0 =>			
					D_OUT <= xDATA(0);
					STATE <= DAT_ELSE;	
--------------------------------------------------------------------------------	
				when DAT_ELSE =>			
					D_OUT <= '1';
--------------------------------------------------------------------------------	
				when others =>	STATE<=DAT14;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------
end Behavioral;