--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 26 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: DAC_TIMING		  																--
--	Description : 																					--
-- 	DAC timing module 																		--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DAC_TIMING is
   port ( 
			 xCLK    	: in    std_logic;	
			 xREFRESH 	: in    std_logic;
			 xCLR_ALL 	: in    std_logic;			 
			 xSCLK	 	: out    std_logic;
			 xNSYNC	 	: out    std_logic;
			 xUPDATE	 	: out    std_logic;
			 xEN_A		: out    std_logic;
			 xEN_B		: out    std_logic;
			 xEN_C		: out    std_logic;
			 xEN_D		: out    std_logic;
			 xS 		  	: out   std_logic_vector (1 downto 0));
end DAC_TIMING;

architecture Behavioral of DAC_TIMING is
	type STATE_TYPE is ( IDLE, COUNTER, CLR_CNT); 
	signal STATE 		   : STATE_TYPE;
	signal xCNT				: std_logic_vector(15 downto 0);	
	signal CNT				: std_logic_vector(15 downto 0);	
	signal SCLK				: std_logic;
	signal NSYNC			: std_logic;
	signal UPDATE			: std_logic;
	signal EN_A				: std_logic;
	signal EN_B				: std_logic;
	signal EN_C				: std_logic;
	signal EN_D				: std_logic;
	signal xSTART			: std_logic;
	signal xDONE			: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component INV
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component BUF16
      port ( I  : in    std_logic_vector(15 downto 0); 
             O  : out   std_logic_vector(15 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	xBUF_SCLK : BUF 
	port map (
		I  => SCLK,
		O  => xSCLK);		
--------------------------------------------------------------------------------
	xBUF_NSYNC : BUF 
	port map (
		I  => NSYNC,
		O  => xNSYNC);		
--------------------------------------------------------------------------------
	xBUF_UPDATE : BUF 
	port map (
		I  => xCNT(6),
		O  => xUPDATE);		
--------------------------------------------------------------------------------
	xBUF_EN_A : BUF 
	port map (
		I  => EN_A,
		O  => xEN_A);		
--------------------------------------------------------------------------------
	xBUF_EN_B : BUF 
	port map (
		I  => EN_B,
		O  => xEN_B);		
--------------------------------------------------------------------------------
	xBUF_EN_C : BUF 
	port map (
		I  => EN_C,
		O  => xEN_C);		
--------------------------------------------------------------------------------
	xBUF_EN_D : BUF 
	port map (
		I  => EN_D,
		O  => xEN_D);	
--------------------------------------------------------------------------------
	xBUF_S0 : BUF 
	port map (
		I  => xCNT(7),
		O  => xS(0));	
--------------------------------------------------------------------------------
	xBUF_S1 : BUF 
	port map (
		I  => xCNT(8),
		O  => xS(1));	
--------------------------------------------------------------------------------
	xINV_SCLK : INV 
	port map (
		I  => xCNT(0),
		O  => SCLK);			
--------------------------------------------------------------------------------
	xINV_NSYNC : INV 
	port map (
		I  => xCNT(6),
		O  => NSYNC);			
--------------------------------------------------------------------------------
	process(xCLR_ALL,xREFRESH)
	begin
		if xCLR_ALL = '1' or xDONE = '1' then
			xSTART <= '0';
		elsif falling_edge(xREFRESH) then
			xSTART <= '1';
		end if;
	end process;
--------------------------------------------------------------------------------
	process(xCLR_ALL,xCLK,xSTART)
	begin
		if xCLR_ALL = '1' then 
			STATE <= IDLE;
			CNT <= (others=>'0');
			xDONE <= '0';
			UPDATE <= '0';
		elsif rising_edge(xCLK) then
			xDONE <= '0';
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when IDLE =>			
					if xSTART = '1' then
						UPDATE <= '1';
						STATE <= COUNTER;		
					end if;
--------------------------------------------------------------------------------	
				when COUNTER =>			
					CNT <= CNT + 1;
					if CNT = 512 then
						CNT <= (others=>'0');
						STATE <= CLR_CNT;		
					end if;
--------------------------------------------------------------------------------	
				when CLR_CNT =>			
					xDONE <= '1';
					UPDATE <= '0';
					STATE <= IDLE;
--------------------------------------------------------------------------------	
				when others =>	STATE<=IDLE;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------	
	process(UPDATE,xCNT)
	begin
		if UPDATE = '1' and xCNT(8) = '0' and xCNT(7) = '0' then
			EN_A <= '1';
			EN_B <= '0';
			EN_C <= '0';
			EN_D <= '0';
		elsif UPDATE = '1' and xCNT(8) = '0' and xCNT(7) = '1' then
			EN_A <= '0';
			EN_B <= '1';
			EN_C <= '0';
			EN_D <= '0';
		elsif UPDATE = '1' and xCNT(8) = '1' and xCNT(7) = '0' then
			EN_A <= '0';
			EN_B <= '0';
			EN_C <= '1';
			EN_D <= '0';
		elsif UPDATE = '1' and xCNT(8) = '1' and xCNT(7) = '1' then
			EN_A <= '0';
			EN_B <= '0';
			EN_C <= '0';
			EN_D <= '1';
		else
			EN_A <= '0';
			EN_B <= '0';
			EN_C <= '0';
			EN_D <= '0';
		end if;
	end process;	
--------------------------------------------------------------------------------							
	xBUF_CNT : BUF16 
	port map (
		I(15 downto 0)=> CNT(15 downto 0),
		O(15 downto 0)=> xCNT(15 downto 0));
--------------------------------------------------------------------------------
end Behavioral;