--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 12 JULY 2007																			--
-- Project name: BLAB1 firmware																--
--	Module name: RCO_INT		  																	--
--	Description : 																					--
-- 	Integrates the RCO from LAB3															--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity RCO_INT is
   port ( 
			 xCLK    		: in    std_logic;
			 xREFRESH  		: in    std_logic;			 
			 xRCO	 			: in    std_logic;
			 xINT 		  	: out   std_logic_vector (15 downto 0));
end RCO_INT;

architecture Behavioral of RCO_INT is
	type STATE_TYPE is ( PHASE0, PHASE1, PHASE2, PHASE3,
								RPHASE0, RPHASE1, RPHASE2, RPHASE3); 
	signal STATE			: STATE_TYPE := PHASE0;
	signal RSTATE			: STATE_TYPE := RPHASE0;
	signal INT				: std_logic_vector(15 downto 0) := x"0000";	
	signal WADDR			: std_logic_vector(11 downto 0) := x"000";	
	signal RADDR			: std_logic_vector(11 downto 0) := x"000";	
	signal WSTRB			: std_logic := '0';
	signal RSTRB			: std_logic := '0';
	signal CNT_CLR			: std_logic := '0';
	signal RCO_CNT			: std_logic_vector(3 downto 0) := x"0";	
	signal READ_RAM		: std_logic_vector(3 downto 0);
	signal SUMMING			: std_logic_vector(15 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------	
   component RCO_RAM	
   port ( xRADDR  : in    std_logic_vector (11 downto 0); 
          xWADDR  : in    std_logic_vector (11 downto 0); 
          xR_CLK  : in    std_logic; 
          xR_EN   : in    std_logic; 
          xWRITE  : in    std_logic_vector (3 downto 0); 
          xW_CLK  : in    std_logic; 
          xW_EN   : in    std_logic; 
          xREAD   : out   std_logic_vector (3 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------
	process(xRCO,xCLK)
	begin
		if falling_edge(xCLK) then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when PHASE0 =>		
					WSTRB 	<= '0';
					CNT_CLR 	<= '0';
					if xRCO = '0' then
						STATE <= PHASE1;
					end if;
--------------------------------------------------------------------------------	
				when PHASE1 =>		
					WSTRB 	<= '1';
					CNT_CLR 	<= '0';	
					STATE 	<= PHASE2;
--------------------------------------------------------------------------------	
				when PHASE2 =>		
					WSTRB 	<= '0';
					CNT_CLR 	<= '1';
					WADDR		<= WADDR + 1;		
					STATE 	<= PHASE3;
--------------------------------------------------------------------------------	
				when PHASE3 =>		
					WSTRB 	<= '0';
					CNT_CLR 	<= '0';	
					if xRCO = '1' then
						STATE <= PHASE0;
					end if;
--------------------------------------------------------------------------------	
				when others =>	STATE<=PHASE0;																
			end case;
		end if;
	end process;		
--------------------------------------------------------------------------------							
	process(xRCO,xCLK)
	begin
		if CNT_CLR = '1' then
			RCO_CNT <= x"0";
		elsif rising_edge(xCLK) and xRCO = '1' then	
			RCO_CNT <= RCO_CNT + 1;
		end if;
	end process;
--------------------------------------------------------------------------------							
	xRCO_RAM : RCO_RAM 
	port map (
		xRADDR 	=> RADDR,
		xWADDR	=> WADDR,
		xR_CLK	=> RSTRB,
		xR_EN 	=> '1',
		xWRITE 	=> RCO_CNT,
		xW_CLK	=> WSTRB,
		xW_EN	 	=> '1',
		xREAD		=> READ_RAM);		
--------------------------------------------------------------------------------
	SUMMING <= x"000" & READ_RAM;
--------------------------------------------------------------------------------
	process(xREFRESH,xCLK)
	begin
		if rising_edge(xCLK) then
--------------------------------------------------------------------------------			
			case RSTATE is
--------------------------------------------------------------------------------	
				when RPHASE0 =>		
					RSTRB 	<= '0';
					RADDR		<= x"000";
					if xREFRESH = '0' then
						INT <= x"0000";
						RSTATE <= RPHASE1;
					end if;
--------------------------------------------------------------------------------	
				when RPHASE1 =>		
					RSTRB 	<= '1';
					RSTATE 	<= RPHASE2;
--------------------------------------------------------------------------------	
				when RPHASE2 =>		
					RSTRB 	<= '0';
					RADDR		<= RADDR + 1;
					INT <= INT + SUMMING;
					if RADDR = 4095 then 
						RSTATE <= RPHASE3;	
					else
						RSTATE <= RPHASE1;	
					end if;					
--------------------------------------------------------------------------------	
				when RPHASE3 =>		
					RSTRB 	<= '0';
					if xREFRESH = '1' then
						RSTATE <= RPHASE0;
					end if;
--------------------------------------------------------------------------------	
				when others =>	RSTATE<=RPHASE0;																
			end case;
		end if;
	end process;		
--------------------------------------------------------------------------------
	xBUF_INT : BUF16 
	port map (
		I(15 downto 0)=> INT(15 downto 0),
		O(15 downto 0)=> xINT(15 downto 0));
--------------------------------------------------------------------------------
end Behavioral;