--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 22 OCT 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: BLAB1_PIXx32	  																--
--	Description : 																					--
-- 	32 stored cell block																		--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TARGET_PIXx32 is
port ( 
			 xFCLK			: in    std_logic;
			 xRAMP			: in    std_logic;
			 xDONE			: in    std_logic;
			 xRESET_RAMP		: in    std_logic;
			 xCLR_ALL		: in    std_logic;
			 xTOUT			: in    std_logic_vector(15 downto 0);
			 xBOUT			: in    std_logic_vector(15 downto 0);
			 xWADDR			: in    std_logic_vector(15 downto 0);
			 xT_DATA			: out	  std_logic_vector(15 downto 0);
			 xB_DATA			: out   std_logic_vector(15 downto 0));
end TARGET_PIXx32;

architecture Behavioral of TARGET_PIXx32 is
	type 	 CNT_TYPE is array(1 downto 0, 15 downto 0) of std_logic_vector(11 downto 0);
	signal xCNT_RISING				: CNT_TYPE;
	signal xCNT_FALLING				: CNT_TYPE;
	type 	 CHECK_TYPE is array(1 downto 0) of std_logic_vector(31 downto 0);
	signal xCHECK_RISING		: CHECK_TYPE;	
	signal xCHECK_FALLING	: CHECK_TYPE;	
	signal CH_NUMBER  		: integer range 31 downto 0;	
--------------------------------------------------------------------------------
	signal xGCC1		: std_logic_vector(11 downto 0);	
	signal xGCC2		: std_logic_vector(11 downto 0);	
	signal xREF			: std_logic_vector(11 downto 0);	
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------	
	component GCC_REF
   port ( 
			 xFCLK    		: in    std_logic;
			 xHIT    		: in    std_logic;
			 xREF				: out	  std_logic_vector(11 downto 0);
			 xGCC1			: out	  std_logic_vector(11 downto 0);
			 xGCC2			: out	  std_logic_vector(11 downto 0));			
   end component;
--------------------------------------------------------------------------------	
   component TARGET_PIXx16
	port ( 
			 xRAMP			: in    std_logic;
			 xREF    		: in    std_logic_vector (11 downto 0);
			 xDAT				: in    std_logic_vector (15 downto 0);
			 xGCC1			: in    std_logic_vector (11 downto 0);
			 xGCC2			: in    std_logic_vector (11 downto 0);
			 xADDR			: in    std_logic_vector (3 downto 0);
			 xOUT				: out   std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------	
--	xGCC_REF : GCC_REF 
--	port map (
--		xFCLK 			=> xFCLK,
--		xHIT 				=> xRAMP,
--		xREF	 			=> xREF,
--		xGCC1 			=> xGCC1,
--		xGCC2	 			=> xGCC2);	
----------------------------------------------------------------------------------
--	xTARGET_PIXx16_A : TARGET_PIXx16 
--	port map (
--		xRAMP 			=> xRAMP,
--		xREF 				=> xREF,
--		xDAT	 			=> xTOUT,
--		xADDR(3 downto 0)	=> xWADDR(3 downto 0),
--		xGCC1 			=> xGCC1,
--		xGCC2 			=> xGCC2,
--		xOUT	 			=> xT_DATA);			
----------------------------------------------------------------------------------
--	xTARGET_PIXx16_B : TARGET_PIXx16 
--	port map (
--		xRAMP 			=> xRAMP,
--		xREF 				=> xREF,
--		xDAT	 			=> xBOUT,
--		xADDR(3 downto 0)	=> xWADDR(3 downto 0),
--		xGCC1 			=> xGCC1,
--		xGCC2 			=> xGCC2,
--		xOUT	 			=> xB_DATA);		
--------------------------------------------------------------------------------
	GEN_TOP_RISING : for I in 15 downto 0 generate	
	process(xFCLK,xCLR_ALL,xDONE)
	begin
		if xCLR_ALL = '1' or xDONE = '1' or xRESET_RAMP = '1' then
			xCNT_RISING(0,I) <= x"000";
			xCHECK_RISING(0)(I) <= '0';
		elsif rising_edge(xFCLK) and xRAMP = '1' and xCHECK_RISING(0)(I) = '0' then
			xCNT_RISING(0,I) <= xCNT_RISING(0,I) + 1;
			if xTOUT(I) = '1' then
				xCHECK_RISING(0)(I) <= '1';
			end if;
		end if;
	end process;
	end generate GEN_TOP_RISING;	
--------------------------------------------------------------------------------
	GEN_BOTTOM_RISING : for I in 15 downto 0 generate	
	process(xFCLK,xCLR_ALL,xDONE)
	begin
		if xCLR_ALL = '1' or xDONE = '1' or xRESET_RAMP = '1' then
			xCNT_RISING(1,I) <= x"000";
			xCHECK_RISING(1)(I) <= '0';
		elsif rising_edge(xFCLK) and xRAMP = '1' and xCHECK_RISING(1)(I) = '0' then
			xCNT_RISING(1,I) <= xCNT_RISING(1,I) + 1;
			if xBOUT(I) = '1' then
				xCHECK_RISING(1)(I) <= '1';
			end if;
		end if;
	end process;
	end generate GEN_BOTTOM_RISING;	
--------------------------------------------------------------------------------
	GEN_TOP_FALLING : for I in 15 downto 0 generate	
	process(xFCLK,xCLR_ALL,xDONE)
	begin
		if xCLR_ALL = '1' or xDONE = '1' or xRESET_RAMP = '1' then
			xCNT_FALLING(0,I) <= x"000";
			xCHECK_FALLING(0)(I) <= '0';
		elsif falling_edge(xFCLK) and xRAMP = '1' and xCHECK_FALLING(0)(I) = '0' then
			xCNT_FALLING(0,I) <= xCNT_FALLING(0,I) + 1;
			if xTOUT(I) = '1' then
				xCHECK_FALLING(0)(I) <= '1';
			end if;
		end if;
	end process;
	end generate GEN_TOP_FALLING;	
--------------------------------------------------------------------------------
	GEN_BOTTOM_FALLING : for I in 15 downto 0 generate	
	process(xFCLK,xCLR_ALL,xDONE)
	begin
		if xCLR_ALL = '1' or xDONE = '1' or xRESET_RAMP = '1' then
			xCNT_FALLING(1,I) <= x"000";
			xCHECK_FALLING(1)(I) <= '0';
		elsif falling_edge(xFCLK) and xRAMP = '1' and xCHECK_FALLING(1)(I) = '0' then
			xCNT_FALLING(1,I) <= xCNT_FALLING(1,I) + 1;
			if xBOUT(I) = '1' then
				xCHECK_FALLING(1)(I) <= '1';
			end if;
		end if;
	end process;
	end generate GEN_BOTTOM_FALLING;	
--------------------------------------------------------------------------------
	CH_NUMBER	<= conv_integer(xWADDR(4 downto 0));	
	xT_DATA <= xCNT_RISING(0,CH_NUMBER) + xCNT_FALLING(0,CH_NUMBER);
	xB_DATA <= xCNT_RISING(1,CH_NUMBER) + xCNT_FALLING(1,CH_NUMBER);
--------------------------------------------------------------------------------
end Behavioral;